#include "render.h"
#include "QTime"
#include "qrect.h"
#include "qfont.h"
#include "qfile.h"
#include "qdatastream.h"
#include "qmessagebox.h"
#include "QtCore"
#include "qdebug.h"

#ifndef GL_MULTISAMPLE
#define GL_MULTISAMPLE  0x809D
#endif

GLWidget::GLWidget(QWidget *parent)
    : QGLWidget(QGLFormat(QGL::SampleBuffers), parent)
{
        QTime midnight(0, 0, 0);
         qsrand(midnight.secsTo(QTime::currentTime()));

         setAutoFillBackground(false);
         startTimer(30);
         setWindowTitle(tr("Mappy Engine"));


         //MappySaver m;
         logo = QPixmap(":/images/data/logo_transparent.png");



}


GLWidget::~GLWidget()
{
}



void GLWidget::mouseMoveEvent(QMouseEvent *event)
 {

}

void GLWidget::mousePressEvent(QMouseEvent *event)
{

}

void GLWidget::paintEvent(QPaintEvent *event)
 {
    QPainter painter(this);
    painter.setRenderHint(QPainter::Antialiasing);

    QFont font = painter.font();
    font.setPointSize(font.pointSize() * 2);

    painter.setFont(font);


    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

     makeCurrent();
     glMatrixMode(GL_MODELVIEW);
     glPushMatrix();

     //QColor qtGreen = QColor::fromCmykF(0.40, 0.0, 1.0, 0.0);

     QColor violet = QColor::fromRgb(153,204,255);
     qglClearColor(violet.dark());

     glShadeModel(GL_SMOOTH);
     glEnable(GL_DEPTH_TEST);
     glEnable(GL_CULL_FACE);
     glEnable(GL_LIGHTING);
     glEnable(GL_LIGHT0);
     glEnable(GL_MULTISAMPLE);
     static GLfloat lightPosition[4] = { 0.5, 5.0, 7.0, 1.0 };
     glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);


         glShadeModel(GL_FLAT);
         glDisable(GL_CULL_FACE);
         glDisable(GL_DEPTH_TEST);
         glDisable(GL_LIGHTING);

         glMatrixMode(GL_MODELVIEW);
         glPopMatrix();



          //GL Overlay
         painter.drawPixmap(QPoint(mappa.Tiles[0].x,mappa.Tiles[0].y),Tileset,QRect(0,0,32,32));

          x++;


          //Draw Tiles
          for(int i = 0; i < mappa.Tiles.count(); i++)
          {
              painter.drawPixmap(mappa.Tiles[i].x,mappa.Tiles[i].y,Tileset,mappa.Tiles[i].rect.x(),mappa.Tiles[i].rect.y(),mappa.Tiles[i].rect.width()- mappa.Tiles[i].rect.x() + 32,mappa.Tiles[i].rect.height() - mappa.Tiles[i].rect.y() + 32);
          }



          //logo
          painter.drawPixmap(QRect(this->width()-200,this->height()-200,200,200),logo);
          painter.end();


         //swapBuffers();
        // update();  //Soo Fast! disabled...
        // updateGL();

}


void GLWidget::initializeGL()
 {
    glEnable(GL_MULTISAMPLE);


     //logo = new QtLogo(this);
    // logo->setColor(qtGreen.dark());

    static const char *vertexShaderSource =
        "attribute highp vec4 posAttr;\n"
        "attribute lowp vec4 colAttr;\n"
        "varying lowp vec4 col;\n"
        "uniform highp mat4 matrix;\n"
        "void main() {\n"
        "   col = colAttr;\n"
        "   gl_Position = matrix * posAttr;\n"
        "}\n";

    static const char *fragmentShaderSource =
        "varying lowp vec4 col;\n"
        "void main() {\n"
        "   gl_FragColor = col;\n"
        "}\n";


 }


//! Initialization
void GLWidget::glInit()
{
    //Loading First Asset

    QString path = QDir::currentPath();
    path.append(QString("/data/Map/%1").arg(levelname));

    //Recupero il file mappa
    QFile file(path);
    if(!file.open(QIODevice::ReadOnly))
    {
        QMessageBox::information(this,tr("Errore"),tr("File not found"),QMessageBox::Ok);

    }else
    {
        QDataStream in(&file);
        in.setVersion(QDataStream::Qt_4_8);

        //Load all pice of tiles
        in >> mappa.Tiles;
        file.flush();


        QByteArray array;
        in >> array;

        QString tilename = QString::fromLocal8Bit(array);
        Tileset.load(tilename);


    }


}

void GLWidget::resizeGL(int width, int height)
 {
     setupViewport(width, height);
 }


void GLWidget::timerEvent(QTimerEvent *)
{
    update();
}

void GLWidget::setupViewport(int width, int height)
{
    int side = qMin(width, height);
    glViewport((width - side) / 2, (height - side) / 2, side, side);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
#ifdef QT_OPENGL_ES
    glOrthof(-0.5, +0.5, -0.5, 0.5, 4.0, 15.0);
#else
    glOrtho(-0.5, +0.5, -0.5, 0.5, 4.0, 15.0);
#endif
    glMatrixMode(GL_MODELVIEW);
}
